/* icmplog.h of iplog v1.7 -- eric@ojnk.org */
#ifndef __ICMPLOG_H
#define __ICMPLOG_H

static char *protolookup(int);

struct packet {
	struct ip ip;
	struct icmp icmp;
	struct tcphdr tcp;
};

static void print_router_advertisement(struct packet);
static void print_timestamp_reply(struct packet);
static void print_address_reply(struct packet);
static void print_redirect(struct packet);

#define ICMP_SIZE	(sizeof(struct ip) + 8)

#define proto	pkt.icmp.icmp_ip.ip_p
#define gw		pkt.icmp.icmp_gwaddr
#define src		pkt.ip.ip_src
#define dest	pkt.icmp.icmp_ip.ip_dst
#define orig	pkt.icmp.icmp_ip.ip_src

static const char *codes[] =
{
	"echo reply from %s sz=%u(+%u)",	/* ICMP_ECHO_REPLY */
	"undefined icmp type 1 recieved from %s(?)",	/* ? */
	"undefined icmp type 2 recieved from %s(?)",	/* ? */
	"destination unreachable from %s",	/* ICMP_DEST_UNREACHABLE */
	"source quench from %s",	/* ICMP_SOURCE_QUENCH */
	"redirect message from %s",	/* ICMP_REDIRECT */
	"undefined icmp type 6 recieved from %s(?)",	/* ? */
	"undefined icmp type 7 recieved from %s(?)",	/* ? */
	"echo from %s sz=%u(+%u)",	/* ICMP_ECHO */
	"router advertisement from %s",		/* ICMP_ROUTER_ADVERT (rfc1256) */
	"router solicitation from %s",	/* ICMP_ROUTER_SOLICIT (rfc1256) */
	"time exceeded (%s)",		/* ICMP_TIME_EXCEEDED */
	"parameter problem from %s (ptr %d)",	/* ICMP_PARAMETER_PROBLEM */
	"timestamp request from %s",	/* ICMP_TIMESTAMP */
	"timestamp reply from %s (%s)",		/* ICMP_TIMESTAMP_REPLY */
	"info request from %s",		/* ICMP_INFO_REQUEST */
	"info reply from %s",		/* ICMP_INFO_REPLY */
	"address mask request from %s",		/* ICMP_ADDRESS */
	"address mask reply from %s (%s)"	/* ICMP_ADDRESS_REPLY */
};

static const char *unreach[] =
{
	"%s: network is unreachable [%s: dp=%u sp=%u]",		/* ICMP_NET_UNREACH */
	"%s: host is unreachable [%s: dp=%u sp=%u]",	/* ICMP_HOST_UNREACH */
	"%s: %s: protocol is unreachable",	/* ICMP_PROT_UNREACH */
	"%s: %s port is unreachable [dp=%u sp=%u]",		/* ICMP_PORT_UNREACH */
	"%s: fragmentation needed (IP_DF set)",		/* ICMP_FRAG_NEEDED */
	"%s: source route failed [%s: dp=%u sp=%u]",	/* ICMP_SR_FAILED */
	"%s: network is unreachable(?) [%s: dp=%u sp=%u]",	/* ICMP_NET_UNKNOWN */
	"%s: host is unreachable(?) [%s: dp=%u sp=%u]",		/* ICMP_HOST_UNKNOWN */
	"%s: host is isolated from source",		/* ICMP_HOST_ISOLATED */
	"%s: network is unreachable (admin prohibited) ",	/* ICMP_NET_ANO */
	"%s: host is unreachable (admin prohibited)",	/* ICMP_HOST_ANO */
	"%s: network is unreachable (tos is prohibited)",	/* ICMP_NET_UNR_TOS */
	"%s: host is unreachable (tos is prohibited)",	/* ICMP_HOST_UNR_TOS */
	"%s: packet filtered [%s: dp=%u sp=%u]",	/* ICMP_PKT_FILTERED */
	"%s: precedence violation",	/* ICMP_PREC_VIOLATION */
	"%s: precedence cutoff"		/* ICMP_PREC_CUTOFF */
};

static const char *redir[] =
{
	"(%s) redirect %s to network %s",	/* ICMP_REDIR_NET */
	"(%s) redirect %s to host %s",	/* ICMP_REDIR_HOST */
	"(%s) redirect-tos %s to network %s",	/* ICMP_REDIR_NETTOS */
	"(%s) redirect-tos %s to host %s"	/* ICMP_REDIR_HOSTTOS */
};

#endif /* __ICMPLOG_H */
