/* smurf.h of iplog v1.7 -- eric@ojnk.org */
#include "config.h"
#ifdef DETECT_SMURF
#include "common.h"
#include <arpa/inet.h>

#define HDR_SIZE sizeof(struct ip)

#define SMURF_WAIT	60

#define THRESH_PPS	25			/* packets per second */
#define THRESH_KBPS	20			/* kilobytes per second */

time_t last = 0, lasta = 0;
float kbps = 0.0;
int count = 0, ignore_count = 0, smurf_mode = 0, pps = 0;

u_long *ips, *ignore_ips;

void sigalrm(int stat)
{
	int freed = 0;

	if (smurf_mode) {
		if (count > 0) {
			realloc(ips, 0);
			freed += sizeof(u_long) * count;
		}
		if (ignore_count > 0) {
			realloc(ignore_ips, 0);
			freed += sizeof(u_long) * ignore_count;
		}
		syslog(LEVEL, "No longer in smurf mode (freed %d bytes memory)", freed);
	}
	signal(SIGALRM, sigalrm);
}

int comp(const void *a, const void *b)
{
	return *(int *)a - *(int *)b;
}

/* based on buglord's SmurfLog, but modified heavily to work in iplog */
int smurf(struct gen_packet *pkt)
{
	time_t current;
	u_long the_ip;
	u_char *p = NULL;

	time(&current);

	if (smurf_mode) {
		p = (unsigned char *)&pkt->ip.ip_src.s_addr;

		p[3] = 0;

		the_ip = pkt->ip.ip_src.s_addr;

		if (bsearch(&the_ip, ips, count, sizeof(int *), comp) == NULL) {
			if ((ips = realloc(ips, sizeof(u_long) * (count + 1))) == NULL)
				fatal("realloc: out of memory");

			ips[count++] = the_ip;
			qsort(ips, count, sizeof(u_long), comp);
		} else if (bsearch(&the_ip, ignore_ips, ignore_count, sizeof(u_long), comp) == NULL) {
			syslog(LEVEL, "smurf attack: %s/24 (%d bytes)", inet_ntoa(pkt->ip.ip_src), UNFIX(pkt->ip.ip_len) - HDR_SIZE);

			if ((ignore_ips = realloc(ignore_ips, sizeof(u_long) * (ignore_count + 1))) == NULL)
				fatal("realloc: out of memory");

			ignore_ips[ignore_count++] = the_ip;
			qsort(ignore_ips, ignore_count, sizeof(u_long), comp);
		}
		if ((lasta + 5) <= time(NULL)) {
			alarm(SMURF_WAIT);
			lasta = current;
		}
		return (0);
	} else {
		kbps += UNFIX(pkt->ip.ip_len) / 1024.0;
		pps++;

		if ((current - last) >= 1 || (current < last)) {
			pps = 0;
			kbps = 0.0;
			last = current;
			return (1);
		} else if (pps >= THRESH_PPS && kbps >= THRESH_KBPS) {
			smurf_mode = 1;
			syslog(LEVEL, "threshold met (%ld pkt/s, %4.2fkb/s)", pps, kbps);
			last = current;
			return (0);
		}
		last = current;
		return (1);
	}
}

#endif /* DETECT_SMURF */
