/* udplog.c of iplog v1.7 -- eric@ojnk.org */
#include <unistd.h>
#include <time.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <string.h>
#include <stdio.h>
#include <signal.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netdb.h>
#include <fcntl.h>
#include <ctype.h>
#include <arpa/inet.h>
#include "config.h"
#include "common.h"
#include "udplog.h"

u_long ignore[MAX_NAMESERVERS];

int main(void)
{
	struct udp_packet pkt;
	int sock, i, bad = 0;

#ifdef DETECT_TRACEROUTE
	time_t lasttr = 0;
	u_long last = 0;

#endif

	if ((sock = socket(AF_INET, SOCK_RAW, IPPROTO_UDP)) < 0) {
		perror("socket");
		exit(-1);
	}
	openlog("udplog", LOGGING_OPTIONS, LOGGING_FACILITY);
	signal(SIGHUP, reread_ns);

	fork_to_background();
	initialize_lockfile("udplog");
	drop_privileges();
	readns(ignore);

	for (;;) {
		read(sock, (struct header *)&pkt, sizeof(pkt));

		if (UNFIX(pkt.udp.source) == 53) {
			for (i = 0, bad = 0; ignore[i] != (-1L); i++) {
				if (pkt.ip.ip_src.s_addr == ignore[i]) {
					bad = 1;
					break;
				}
			}
		} else
			bad = 0;

		if (!bad) {
#ifdef DETECT_SMURF
			if ((UNFIX(pkt.udp.dest) == 7 || UNFIX(pkt.udp.dest) == 19) && !smurf((struct gen_packet *)&pkt))
				continue;
#endif

#ifdef DETECT_TRACEROUTE
			if (((dp > 32000) && (dp < 40000)) && (sp > 30000) && (ttl == 1)) {
				if ((last != pkt.ip.ip_src.s_addr) || (time(NULL) >= (lasttr + 15))) {
					syslog(LEVEL, "traceroute from %s (%s)", hostlookup(pkt.ip.ip_src.s_addr), inet_ntoa(pkt.ip.ip_src));
					time(&lasttr);
					last = pkt.ip.ip_src.s_addr;
				}
			} else
#endif
				syslog(LEVEL, "dgram to %s from %s:%u sz=%u(+%u)", servlookup(pkt.udp.dest), inet_ntoa(pkt.ip.ip_src), UNFIX(pkt.udp.source), UNFIX(pkt.ip.ip_len) - UDP_SIZE, UDP_SIZE);
		}
		memset((char *)&pkt, 0, sizeof(pkt));
	}
	close(sock);
}

static char *servlookup(u_short port)
{
	struct servent *se;

	if ((se = getservbyport(port, "udp")) != NULL)
		return (se->s_name);
	else {
		static char buff[16];

		sprintf(buff, "port %u", UNFIX(port));
		return (buff);
	}
}

#ifdef DETECT_TRACEROUTE
static char *hostlookup(u_long addr)
{
	struct hostent *he;
	struct in_addr i;

	i.s_addr = addr;
	he = gethostbyaddr((char *)&i, sizeof(struct in_addr), AF_INET);

	if (he != NULL)
		return (he->h_name);
	else
		return (inet_ntoa(i));
}
#endif

void reread_ns(int stat)
{
	syslog(LEVEL, "caught hangup signal -- re-reading nameservers");
	readns(ignore);
	signal(SIGHUP, reread_ns);
}

void readns(u_long * ignore)
{
	FILE *fp;
	int found = 0;
	char line[BUFSIZ], host[18];

	if ((fp = fopen(_PATH_RESCONF, "r")) == NULL)
		fatal("%s: %m", _PATH_RESCONF);

	while ((fgets(line, sizeof(line) - 1, fp) != NULL) && (found != MAX_NAMESERVERS))
		if (sscanf(line, "nameserver %18s", host) == 1)
			if ((ignore[found++] = inet_addr(host)) == (-1L))
				found--;

	(void)fclose(fp);

	syslog(LEVEL, "cached %d nameserver%s", found, (found == 1 ? "" : "s"));
	ignore[found++] = (-1L);

	return;
}
